//Phoenix Julia Shader
//Based of tonfilm's fractal shader

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Color Palette";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};


//User parameters

float2 scale = 1;
float power1 = 2;
float power2 = 0;
float2 center;
float2 offset;
float2 induct;
int iter = 10;


#include "Complex.fxh"

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}



// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------
float4 PS_JuliaPhoenix(vs2ps In): COLOR
{
    float2 z, c;

    float2 y,newz;
    y = float2(0,0);
    newz = float2(0,0);
    


    z = (In.TexCd-0.5) * scale + offset;

    int i;
    c = z;
    bool stop = false;
    for(i=0; i<iter && !stop; i++)
    {      
        newz = Add(Power(z,power1),Multiply(y,induct));
        newz = Add(newz,Multiply(Power(z,power2),center));
        y = z;
        if((newz.x * newz.x + newz.y * newz.y) > 1e20) stop = true;
        z = newz;
    }

    float4 col = tex1D(Samp, (float)(i == iter ? 0 : i) / iter);
    return col;
}



// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------
technique TPhoenixJulia
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_3_0 PS_JuliaPhoenix();
    }
}




technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
